/*
 *  $Id: demo.cpp 9 2008-05-02 08:14:34Z andyman $
 *
 *  Single scale object detection with WaldBoost classifier.
 *
 *  The program takes video from camera, scans the image and displays the
 *  detections.
 */

#include <iostream>
#include <vector>

#include <cv.h>
#include <cxcore.h>
#include <highgui.h>

#include "lrd_engine.h"
#include "faces.h"


using namespace std;


int main()
{
	CvCapture * capture = cvCaptureFromCAM(0);

	if (!capture)
	{
		cerr << "No camera found!" << endl;
		return 0;
	}

	IplImage * grayFrame = 0;  // Input frame converted to grayscale
	IplImage * grayFrameScale = 0;
    TImage img; // this image is actually used for detection
    
    initClassifier(&classifier); // Initialization of a classifier
    
    TRect results[1000];

	cvNamedWindow("Image");

	for (;;)
	{
		if (!cvGrabFrame(capture)) break;
		IplImage * inputFrame = cvRetrieveFrame(capture);

		// Init images
		if (!grayFrame)
		{
			grayFrame = cvCreateImage(cvSize(inputFrame->width, inputFrame->height), IPL_DEPTH_8U, 1);
			grayFrameScale = cvCreateImage(cvSize(320,240), IPL_DEPTH_8U, 1);
            // link img to grayFrame
            img.width = grayFrameScale->width;
            img.height = grayFrameScale->height;
            img.widthStep = grayFrameScale->widthStep;
            img.imageData = grayFrameScale->imageData;
            setClassifierImageSize(&classifier, img.widthStep);
		}
        
		// flip if needed
        if (inputFrame->origin)
        {
            inputFrame->origin = 0;
            cvFlip(inputFrame);
        }

		cvCvtColor(inputFrame, grayFrame, CV_RGB2GRAY);
        cvResize(grayFrame, grayFrameScale, CV_INTER_CUBIC);
        cvSmooth(grayFrameScale, grayFrameScale);
        
        ///////////////////////////////////
        // Scan the image and draw results
        // Actual detection happens here
        int n = scanImage(&img, &classifier, results, results+1000);
        ///////////////////////////////////

        for (int res = 0; res < n; ++res)
        {
            TRect & r = results[res];
            
            cvRectangle(grayFrameScale, cvPoint(r.x,r.y), cvPoint(r.x+r.width,r.y+r.height), cvScalar(0,0,0), 3);
            cvRectangle(grayFrameScale, cvPoint(r.x,r.y), cvPoint(r.x+r.width,r.y+r.height), cvScalar(255,255,255));
        }

		// Show 
        cvShowImage("Image", grayFrameScale);

		// Boring stuff, 
        char c;
		if ((c = cvWaitKey(10)) == 'q') break;
		if (c == 'T') classifier.threshold += 0.5;
		if (c == 't') classifier.threshold -= 0.5;
		if (c >= 0)
		{
			cout << "t=" << classifier.threshold << endl;
		}

	}

	return 0;
}

