/*
 * GStreamer
 * Copyright (C) 2005 Thomas Vander Stichele <thomas@apestaart.org>
 * Copyright (C) 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
 * Copyright (C) 2010 Jozef Mlich <<user@hostname.org>>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Alternatively, the contents of this file may be used under the
 * GNU Lesser General Public License Version 2.1 (the "LGPL"), in
 * which case the following provisions apply instead of the ones
 * mentioned above:
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/**
 * SECTION:element-abr2
 *
 * FIXME:Describe abr2 here.
 *
 * <refsect2>
 * <title>Example launch line</title>
 * |[
 * gst-launch -v -m fakesrc ! abr2 ! fakesink silent=TRUE
 * ]|
 * </refsect2>
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gst/gst.h>
#include <string.h>
#include <time.h>

#include "gstabr2.h"

#include "faces.h"
#include "lrd_engine.h"

TImage img; // this image is actually used for detection
TRect results[1000];
int frame_counter;
time_t start_time, current_time;



GST_DEBUG_CATEGORY_STATIC (gst_abr2_debug);
#define GST_CAT_DEFAULT gst_abr2_debug

/* Filter signals and args */
enum
{
  /* FILL ME */
  LAST_SIGNAL
};

enum
{
  PROP_0,
  PROP_SILENT
};

/* the capabilities of the inputs and outputs.
 *
 * describe the real formats here.
 */
static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
    GST_PAD_SINK,
    GST_PAD_ALWAYS,
    GST_STATIC_CAPS ("ANY")
    );

static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
    GST_PAD_SRC,
    GST_PAD_ALWAYS,
    GST_STATIC_CAPS ("video/x-raw-gray")
    );

GST_BOILERPLATE (Gstabr2, gst_abr2, GstElement, GST_TYPE_ELEMENT);

static void gst_abr2_set_property (GObject * object, guint prop_id, const GValue * value, GParamSpec * pspec);
static void gst_abr2_get_property (GObject * object, guint prop_id, GValue * value, GParamSpec * pspec);

static gboolean gst_abr2_set_caps (GstPad * pad, GstCaps * caps);
static GstFlowReturn gst_abr2_chain (GstPad * pad, GstBuffer * buf);

/* GObject vmethod implementations */

static void
gst_abr2_base_init (gpointer gclass)
{
  GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);

  gst_element_class_set_details_simple(element_class,
    "abr2",
    "Adaboost runtime",
    "Adaboost Facedetector - detects frontal faces in image",
    "Jozef Mlich <<user@hostname.org>>");

  gst_element_class_add_pad_template (element_class,
      gst_static_pad_template_get (&src_factory));
  gst_element_class_add_pad_template (element_class,
      gst_static_pad_template_get (&sink_factory));

}

/* initialize the abr2's class */
static void gst_abr2_class_init (Gstabr2Class * klass) {
  GObjectClass *gobject_class;
  GstElementClass *gstelement_class;

  gobject_class = (GObjectClass *) klass;
  gstelement_class = (GstElementClass *) klass;

  gobject_class->set_property = gst_abr2_set_property;
  gobject_class->get_property = gst_abr2_get_property;

  g_object_class_install_property (gobject_class, PROP_SILENT, g_param_spec_boolean ("silent", "Silent", "Produce verbose output ?",
          FALSE, G_PARAM_READWRITE));


}

/* initialize the new element
 * instantiate pads and add them to element
 * set pad calback functions
 * initialize instance structure
 */
static void gst_abr2_init (Gstabr2 * filter, Gstabr2Class * gclass) {
  filter->sinkpad = gst_pad_new_from_static_template (&sink_factory, "sink");
  gst_pad_set_setcaps_function (filter->sinkpad, GST_DEBUG_FUNCPTR(gst_abr2_set_caps));
  gst_pad_set_getcaps_function (filter->sinkpad, GST_DEBUG_FUNCPTR(gst_pad_proxy_getcaps));
  gst_pad_set_chain_function (filter->sinkpad, GST_DEBUG_FUNCPTR(gst_abr2_chain));

  filter->srcpad = gst_pad_new_from_static_template (&src_factory, "src");
  gst_pad_set_getcaps_function (filter->srcpad, GST_DEBUG_FUNCPTR(gst_pad_proxy_getcaps));

  gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
  gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
  filter->silent = FALSE;


}

static void
gst_abr2_set_property (GObject * object, guint prop_id,
    const GValue * value, GParamSpec * pspec)
{
  Gstabr2 *filter = GST_ABR2 (object);

  switch (prop_id) {
    case PROP_SILENT:
      filter->silent = g_value_get_boolean (value);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
  }
}

static void
gst_abr2_get_property (GObject * object, guint prop_id,
    GValue * value, GParamSpec * pspec)
{
  Gstabr2 *filter = GST_ABR2 (object);

  switch (prop_id) {
    case PROP_SILENT:
      g_value_set_boolean (value, filter->silent);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
  }
}

inline void drawRect(TImage* img, const int x, const int y, const int w, const int h, const int llen, int color) {
  int i;
  for (i = x; i < x + w; i++) {
    img->imageData [i + y * llen] = color;
    img->imageData [i + (y + h) * llen] = color;
  }

  for (i = y; i < y+h; i++) {
    img->imageData [i * llen + x] = color;
    img->imageData [i * llen + x + w] = color;
  }
  
}

/* GstElement vmethod implementations */

/* this function handles the link with other elements */
static gboolean
gst_abr2_set_caps (GstPad * pad, GstCaps * caps)
{
  Gstabr2 *filter;
  GstPad *otherpad;

  filter = GST_ABR2 (gst_pad_get_parent (pad));
  otherpad = (pad == filter->srcpad) ? filter->sinkpad : filter->srcpad;
  gst_object_unref (filter);

  return gst_pad_set_caps (otherpad, caps);
}

/* chain function
 * this function does the actual processing
 */
static GstFlowReturn
gst_abr2_chain (GstPad * pad, GstBuffer * inbuf)
{
  Gstabr2 *filter;

  filter = GST_ABR2 (GST_OBJECT_PARENT (pad));

  int width, height;
  GstStructure *structure;

  structure = gst_caps_get_structure (pad->caps, 0);
  gst_structure_get_int (structure, "width", &width);
  gst_structure_get_int (structure, "height", &height);


  GstBuffer* outbuf;

  gst_pad_alloc_buffer(filter->srcpad, GST_BUFFER_OFFSET_NONE, GST_BUFFER_SIZE(inbuf), GST_BUFFER_CAPS(inbuf), &outbuf);
  memcpy( GST_BUFFER_DATA (outbuf), GST_BUFFER_DATA(inbuf), GST_BUFFER_SIZE(inbuf) );

  img.width = width;
  img.height = height;
  img.widthStep = width;
  img.imageData = (char*) GST_BUFFER_DATA(outbuf);

  setClassifierImageSize(&classifier, img.widthStep);

  int res;
  int n = scanImage(&img, &classifier, results, results+1000);

  if (filter->silent == FALSE) {
    g_print ("width height detections. %d %d %d\n", width, height, n);
  }

  for (res = 0; res < n; ++res) {
    TRect * r = results + res;
    drawRect(&img, r->x, r->y, r->width, r->height, width, 0);
    drawRect(&img, r->x-1, r->y-1, r->width+2, r->height+2, width, 255);

    if (filter->silent == FALSE) {
      g_print("%d %d %d %d\n", r->x, r->y, r->width, r->height);
    }
  }

//    drawRect(&img, 0, 0, 100, 100, width);


  GST_BUFFER_TIMESTAMP ( outbuf ) = GST_BUFFER_TIMESTAMP ( inbuf );

  current_time = time(NULL);
  frame_counter++;

  printf("%d %.0f\n", frame_counter, difftime(current_time, start_time));

  return gst_pad_push (filter->srcpad,  outbuf  );
}


/* entry point to initialize the plug-in
 * initialize the plug-in itself
 * register the element factories and other features
 */
static gboolean
abr2_init (GstPlugin * abr2)
{
  /* debug category for fltering log messages
   *
   * exchange the string 'Template abr2' with your description
   */
  GST_DEBUG_CATEGORY_INIT (gst_abr2_debug, "abr2",
      0, "Template abr2");

    initClassifier(&classifier); // Initialization of a classifier


  start_time = time(NULL);
  frame_counter = 0;


  return gst_element_register (abr2, "abr2", GST_RANK_NONE,
      GST_TYPE_ABR2);
}

/* PACKAGE: this is usually set by autotools depending on some _INIT macro
 * in configure.ac and then written into and defined in config.h, but we can
 * just set it ourselves here in case someone doesn't use autotools to
 * compile this code. GST_PLUGIN_DEFINE needs PACKAGE to be defined.
 */
#ifndef PACKAGE
#define PACKAGE "myfirstabr2"
#endif

/* gstreamer looks for this structure to register abr2s
 *
 * exchange the string 'Template abr2' with your abr2 description
 */
GST_PLUGIN_DEFINE (
    GST_VERSION_MAJOR,
    GST_VERSION_MINOR,
    "abr2",
    "Template abr2",
    abr2_init,
    VERSION,
    "LGPL",
    "GStreamer",
    "http://gstreamer.net/"
)
